package org.iphsoft.simon1;

import java.util.List;

import org.iphsoft.simon1.ui.ActivityAnimationUtil;
import org.iphsoft.simon1.util.ImmersiveModeHelper;
import org.iphsoft.simon1.util.MyLog;

import android.os.Bundle;
import android.view.View;
import android.view.View.OnClickListener;
import android.widget.CheckBox;
import android.widget.CompoundButton;
import android.widget.CompoundButton.OnCheckedChangeListener;
import android.widget.Toast;

import com.facebook.Response;
import com.facebook.Session;
import com.facebook.SessionState;
import com.facebook.widget.LikeView;
import com.facebook.widget.LikeView.OnErrorListener;
import com.facebook.widget.LoginButton;
import com.dotemu.ihnmaims.R;

public class FacebookLoginActivity extends FacebookBaseActivity {

	@Override
	protected void onCreate(Bundle savedInstanceState)
	{
		super.onCreate(savedInstanceState);

		mImmersiveModeHelper = new ImmersiveModeHelper(this);

		setContentView(R.layout.facebook_login_dialog);
		
		// Login button
		mLoginButton = (LoginButton)findViewById(R.id.authButton);
		mLoginButton.setPublishPermissions(PUBLISH_PERMISISONS);
		
		// Checkbox
		CheckBox cbEnablePopups = (CheckBox)findViewById(R.id.cbEnablePopups);
		cbEnablePopups.setChecked(AndroidPortAdditions.instance().getEnablePopups());
		cbEnablePopups.setOnCheckedChangeListener(new OnCheckedChangeListener()
		{		
			@Override
			public void onCheckedChanged(CompoundButton buttonView, boolean isChecked)
			{
				AndroidPortAdditions.instance().setEnablePopups(isChecked);
			}
		});

		// Like view
		mLikeView = (LikeView)findViewById(R.id.likeView);
		mLikeView.setObjectId("770649483008536");
		mLikeView.setOnErrorListener(new OnErrorListener()
		{
			
			@Override
			public void onError(Bundle errorBundle)
			{
				MyLog.d("FacebookLoginActivity.onCreate(...).new OnErrorListener() {...}: onError: " + errorBundle);
				
				Toast.makeText(FacebookLoginActivity.this, R.string.like_view_error, Toast.LENGTH_SHORT).show();			
			}
		});
		
		if (!isFacebookLoggedIn())
		{
			mLikeView.setVisibility(View.GONE);
		}
		
		// Back button
		View btnBack = findViewById(R.id.btnBack);
		btnBack.setOnClickListener(new OnClickListener()
		{
			@Override
			public void onClick(View v)
			{
				finish();
				ActivityAnimationUtil.makeActivityFadeTransition(FacebookLoginActivity.this);
			}
		});		
	}

	@Override
	protected void onResume()
	{
		super.onResume();

		mImmersiveModeHelper.supportFullScreenImmersiveMode();
	}


	@Override
	public void onBackPressed()
	{
		super.onBackPressed();

		ActivityAnimationUtil.makeActivityFadeTransition(FacebookLoginActivity.this);
	}
	
	@Override
	protected void onLoggedInStateChange(boolean loggedIn)
	{
		MyLog.d("FacebookLoginActivity: onLoggedInStateChange: " + loggedIn);
		
		if (loggedIn)
		{
			AndroidPortAdditions.instance().clearAllAchievements();
			requestCurrentAchievements();			
		}
		else
		{
			AndroidPortAdditions.instance().clearAllAchievements();
			
		}
	}

	protected void onSessionStateChange(Session session, SessionState state, Exception exception)
	{
		if (state.isOpened())
		{
			MyLog.d("Facebook logged in...");
			
			mLikeView.setVisibility(View.VISIBLE);
		
		} else if (state.isClosed())
		{
			MyLog.d("Facebook logged out...");	
			
			mLikeView.setVisibility(View.GONE);
		}
	}
	
	@Override
	protected void onPublishPermissionsResponse(boolean granted)
	{
		// TODO Auto-generated method stub
	}
	
	@Override
	protected void onAchievementPosted(int index, boolean explicitShare, Response response, String shareMessage)
	{
		// TODO Auto-generated method stub
	}
	
	@Override
	protected void onCurrentAchievementsResponse(List<Integer> ids)
	{
		AndroidPortAdditions.instance().setAchievementIds(ids);
	}

	private ImmersiveModeHelper mImmersiveModeHelper;
	private LoginButton mLoginButton;
	private LikeView mLikeView;
}
