package org.iphsoft.simon1;

import java.util.List;

import org.iphsoft.simon1.ui.ActivityAnimationUtil;
import org.iphsoft.simon1.util.ImmersiveModeHelper;
import org.iphsoft.simon1.util.MyLog;

import android.content.Intent;
import android.os.Bundle;
import android.os.Parcelable;

import com.facebook.Response;
import com.facebook.Session;
import com.facebook.SessionState;
import com.dotemu.ihnmaims.R;

public class InGameFacebookActivity extends FacebookBaseActivity {

	public static final String EXTRA_RESULT = "InGameFacebookActivity.EXTRA_RESULT";
	
	public enum State
	{
		NOT_LOGGED_IN(0), PERMISSIONS_NOT_GRANTED(1), PERMISSIONS_GRANTED(2), UNKNOWN(-1);
		
		public static State fromValue(int value)
		{
			for (State s : State.values())
			{
				if (s.mValue == value)
				{
					return s;
				}
			}
			
			return UNKNOWN;
		}
		
		private State(int value)
		{
			mValue = value;
		}
		
		private int mValue;
	}
	
	@Override
	protected void onCreate(Bundle savedInstanceState)
	{
		super.onCreate(savedInstanceState);
		
		mImmersiveModeHelper = new ImmersiveModeHelper(this);
		
		setContentView(R.layout.in_game_facebook_activity);
		
		// Check the logged in status
		if (isFacebookLoggedIn())
		{
			mState = State.PERMISSIONS_NOT_GRANTED;
			
			// Just request publish permissions
			if (!requestPublishPermissions())
			{
				goBack();
			}
		}
		else
		{
			// Login to facebook
			if (!login())
			{
				goBack();
			}
		}

	}
	
	@Override
	protected void onResume()
	{
		super.onResume();

		mImmersiveModeHelper.supportFullScreenImmersiveMode();
	}
	
	@Override
	protected void onSessionStateChange(Session session, SessionState state, Exception exception)
	{
		// Log any exception
		if (exception != null)
        {
        	MyLog.e("FacebookBaseActivity: call: Exception: " + exception);
        }
		
		if (mState == State.NOT_LOGGED_IN && state.isOpened())
		{
			// Log in is successful, clear previous achievement cache
			AndroidPortAdditions.instance().clearAllAchievements();
			
			// Check permissions
			if (getPublishPermissionState() == PublishPermissionState.ACCEPTED)
			{
				mState = State.PERMISSIONS_GRANTED;
				goBack();
			}
			else
			{
				mState = State.PERMISSIONS_NOT_GRANTED;
				requestPublishPermissions();
			}
		}
		
		if (state.isClosed())
		{
			// Not logged in
			mState = State.NOT_LOGGED_IN;
			goBack();
		}
		
	}
	
	@Override
	protected void onPublishPermissionsResponse(boolean granted)
	{
		MyLog.d("InGameFacebookActivity: onPublishPermissionsResponse: " + granted);
		
		mState = granted ? State.PERMISSIONS_GRANTED : State.PERMISSIONS_NOT_GRANTED;
		goBack();
	}
	
	@Override
	protected void onLoggedInStateChange(boolean loggedIn)
	{
		// TODO Auto-generated method stub
	}
	
	@Override
	protected void onAchievementPosted(int index, boolean explicitShare, Response response, String shareMessage)
	{
		// TODO Auto-generated method stub
	}
	
	@Override
	protected void onCurrentAchievementsResponse(List<Integer> ids)
	{
		// TODO Auto-generated method stub	
	}
	
	private void goBack()
	{
		setResult(RESULT_OK, new Intent().putExtra(EXTRA_RESULT, mState.mValue));
		finish();
		ActivityAnimationUtil.makeActivityFadeTransition(this);
	}
	
	private State mState = State.NOT_LOGGED_IN;
	
	private ImmersiveModeHelper mImmersiveModeHelper;
}
