package org.iphsoft.simon1.ui;

import java.util.Arrays;

import org.iphsoft.simon1.AndroidPortAdditions;
import org.iphsoft.simon1.AndroidPortAdditions.GameType;
import org.iphsoft.simon1.ui.ToggledExtendedButton.OnToggleListener;
import org.iphsoft.simon1.util.DialogUtils;
import org.iphsoft.simon1.util.MyLog;

import android.content.Context;
import android.content.SharedPreferences;
import android.preference.ListPreference;
import android.preference.Preference;
import android.preference.PreferenceGroup;
import android.preference.PreferenceManager;
import android.view.Gravity;
import android.view.View;
import android.view.ViewGroup.LayoutParams;
import android.widget.AdapterView;
import android.widget.AdapterView.OnItemClickListener;
import android.widget.FrameLayout;
import android.widget.LinearLayout;
import android.widget.RadioGroup;
import android.widget.RadioGroup.OnCheckedChangeListener;
import android.widget.SeekBar;
import android.widget.SeekBar.OnSeekBarChangeListener;
import android.widget.TextView;

import com.dotemu.ihnmaims.R;

public class GameMenuUiHelper {

	public static void createPreferenceFrame(final Context context, PreferenceManager prefManager,
			final Preference pref, FrameLayout parent)
	{
		if (pref instanceof ListPreference)
		{
			// Handle ListPreference
			if (AndroidPortAdditions.GAME_TYPE == GameType.IHNM)
			{
				GameMenuUiHelper.createListViewPreferenceFrame(context, prefManager,
						(ListPreference) pref, parent);
			} else
			{
				GameMenuUiHelper.createListPreferenceFrame(context, prefManager,
						(ListPreference) pref, parent);
			}
		} else if (pref instanceof PreferenceGroup)
		{
			// Handle PreferenceGroup
			GameMenuUiHelper.createPreferenceGroupFrame(context, prefManager,
					(PreferenceGroup) pref, parent);
		} else if (pref instanceof SeekBarPreference)
		{
			// Handle SeekBarPreference
			GameMenuUiHelper.createSeekBarPreferenceFrame(context, prefManager,
					(SeekBarPreference) pref, parent);
		}
	}

	public static void createListViewPreferenceFrame(final Context context,
			PreferenceManager prefManager, final ListPreference listPref, FrameLayout parent)
	{
		// Check the number of entries
		CharSequence[] entries = listPref.getEntries();
		final CharSequence[] entryValues = listPref.getEntryValues();
		CharSequence[] entryDescriptions = null;

		// IHNM special case - add descriptions for control modes
		if (AndroidPortAdditions.GAME_TYPE == GameType.IHNM
				&& listPref.getKey().equals("pref_control_mode"))
		{
			entryDescriptions = new CharSequence[]
			{ context.getString(R.string.control_mode_touch_desc),
					context.getString(R.string.control_mode_classic_desc) };
		}

		// Use a listview layout
		View.inflate(context, R.layout.setting_listview_frame, parent);

		// Populate the layout
		final IHNMListView listView = (IHNMListView) parent.findViewById(R.id.settingListView);

		listView.setEntries(entries, entryDescriptions);

		int checkedIndex = -1;
		for (int i = 0; i < entries.length; ++i)
		{

			// Save the checked item
			if (listPref.getEntry().equals(entries[i]))
			{
				checkedIndex = i;
			}
		}

		// Set the initial checked item
		listView.setItemChecked(checkedIndex, true);

		// Set click listener
		listView.setOnItemClickListener(new OnItemClickListener()
		{
			@Override
			public void onItemClick(AdapterView<?> view, View arg1, int pos, long arg3)
			{
				MyLog.d("GameMenuUiHelper.createListViewPreferenceFrame: onItemSelected: " + pos);

				listPref.setValueIndex(pos);

				onSettingChanged(context, listPref.getKey(), entryValues[pos]);
			}
		});
	}

	public static void createListPreferenceFrame(final Context context,
			PreferenceManager prefManager, final ListPreference listPref, FrameLayout parent)
	{
		// Check the number of entries
		CharSequence[] entries = listPref.getEntries();
		final CharSequence[] entryValues = listPref.getEntryValues();

		if (entries.length > 1)
		{

			// Use a radiogroup layout
			View.inflate(context, R.layout.setting_list_frame, parent);

			// Populate the layout
			int checkedIndex = -1;
			RadioGroup radioGroup = (RadioGroup) parent.findViewById(R.id.settingRadioGroup);
			for (int i = 0; i < entries.length; ++i)
			{
				ExtendedRadioButton radioButton = new ExtendedRadioButton(context);

				// FOTAQ uses a layout workaround because of italic font cutting
				// on Android
				int widthParam = (AndroidPortAdditions.GAME_TYPE == GameType.FOTAQ ? LayoutParams.MATCH_PARENT
						: LayoutParams.WRAP_CONTENT);

				radioButton
						.setLayoutParams(new LayoutParams(widthParam, LayoutParams.WRAP_CONTENT));
				radioButton.setGravity(Gravity.CENTER);
				radioButton.setText(entries[i]);
				radioButton.setId(i);
				radioGroup.addView(radioButton);

				// For Simon 1 German version, since not available in the original game - disable the option for
				// "voice and subtitles"
				if (AndroidPortAdditions.GAME_TYPE == GameType.SIMON1)
				{
					SharedPreferences prefs = PreferenceManager
							.getDefaultSharedPreferences(context);
					if (prefs.getString("pref_language", "en").equals("de"))
					{
						if (entries[i].equals(context
								.getString(R.string.voice_mode_voice_subtitles)))
						{
							radioButton.setEnabled(false);
						}
					}
				}
				
				// For T7G Russian version, since no currently Russian subtitles available - disable the options for
				// 'Voice and Subtitles' and 'Subtitles Only'
				if (AndroidPortAdditions.GAME_TYPE == GameType.T7G)
				{
					SharedPreferences prefs = PreferenceManager
							.getDefaultSharedPreferences(context);
					if (prefs.getString("pref_language", "en").equals("ru"))
					{
						if (entries[i].equals(context
								.getString(R.string.voice_mode_voice_subtitles)) || entries[i].equals(context
										.getString(R.string.voice_mode_subtitles_only)))
						{
							radioButton.setEnabled(false);
						}
					}
				}

				// Hide the "original mt32" setting from unsupported devices
				// TODO: off for now
				/*
				 * if (entries[i].equals(getString(R.string.
				 * music_mode_original_mt32)) &&
				 * !AndroidPortAdditions.instance().canUseMT32Emulation()) {
				 * radioButton.setVisibility(View.GONE); }
				 */

				// Save the checked item
				if (listPref.getEntry().equals(entries[i]))
				{
					checkedIndex = i;
				}
			}

			// Set the initial checked item
			radioGroup.check(checkedIndex);

			// Set click listener
			radioGroup.setOnCheckedChangeListener(new OnCheckedChangeListener()
			{

				@Override
				public void onCheckedChanged(RadioGroup view, int checkedId)
				{
					listPref.setValueIndex(checkedId);

					onSettingChanged(context, listPref.getKey(), entryValues[checkedId]);
				}
			});
		} else
		{
			// Use a toggle button
			View.inflate(context, R.layout.setting_toggle_frame, parent);

			ToggledExtendedButton toggleButton = (ToggledExtendedButton) parent
					.findViewById(R.id.toggleButton);
			toggleButton.setLabels(Arrays.asList(entries));
			for (int i = 0; i < entries.length; ++i)
			{
				if (entries[i].equals(listPref.getEntry()))
				{
					// Set the currently selected setting
					toggleButton.setLabelIndex(i);
				}
			}

			toggleButton.setOnToggleListener(new OnToggleListener()
			{

				@Override
				public void onToggle(int index)
				{
					// Update the setting
					listPref.setValueIndex(index);
				}
			});
		}
	}
	
	public static void onSettingChanged(Context context, CharSequence key, CharSequence newValue)
	{
		// Specific for MT32 option - show a warning message
		if (key.equals("pref_music_mode")
				&& newValue.equals("original_mt32"))
		{
			DialogUtils.createNegativePositiveDialog(context, 0, R.string.ok,
					R.string.mt32_warning_message, null, null).show();
		}

		// Specific for "NONE" option - disable music seekbar
		if (key.equals("pref_music_mode") && sMusicVolSeekbar != null)
		{
			if (newValue.equals("none"))
			{
				sMusicVolSeekbar.setEnabled(false);
			} else
			{
				sMusicVolSeekbar.setEnabled(true);
				sMusicVolSeekbar.setProgress(AndroidPortAdditions.instance().getDefaultMusicVolume());
			}
		}

		// Specific for "subtitle only" option - disable voice
		// seekbar
		if (key.equals("pref_voice_mode") && sSpeechVolSeekbar != null)
		{
			if (newValue.equals("subtitles"))
			{
				sSpeechVolSeekbar.setEnabled(false);
			} else
			{
				sSpeechVolSeekbar.setEnabled(true);
			}
		}

		// Specific for "voice only" option - disable subtitle speed
		// seekbar
		if (key.equals("pref_voice_mode") && sSubtitleSpeedSeekbar != null)
		{
			if (newValue.equals("subtitles"))
			{
				sSubtitleSpeedSeekbar.setVisibility(View.VISIBLE);
				sSubtitleSpeedSeekbar.setEnabled(true);
			} else
			{
				sSubtitleSpeedSeekbar.setVisibility(View.INVISIBLE);
				sSubtitleSpeedSeekbar.setEnabled(false);
			}
		}
	}

	public static void createPreferenceGroupFrame(final Context context,
			PreferenceManager prefManager, final PreferenceGroup prefGroup, FrameLayout parent)
	{
		// Create a vertical scrolling linear layout and populate it with the
		// sub preferences
		View.inflate(context, R.layout.setting_group_frame, parent);

		LinearLayout groupLayout = (LinearLayout) parent.findViewById(R.id.prefGroupLinearLayout);
		for (int i = 0; i < prefGroup.getPreferenceCount(); ++i)
		{
			FrameLayout prefFrame = new FrameLayout(context);
			prefFrame.setLayoutParams(new FrameLayout.LayoutParams(LayoutParams.FILL_PARENT,
					LayoutParams.WRAP_CONTENT, Gravity.CENTER));

			// Create the sub preference frame
			createPreferenceFrame(context, prefManager, prefGroup.getPreference(i), prefFrame);

			// Add it to the group layout
			groupLayout.addView(prefFrame);
		}

	}

	public static void createSeekBarPreferenceFrame(final Context context,
			PreferenceManager prefManager, final SeekBarPreference pref, FrameLayout parent)
	{
		// Create a vertical scrolling linear layout and populate it with the
		// sub preferences
		View.inflate(context, R.layout.setting_seekbar_frame, parent);

		// Set the title
		TextView txtTitle = (TextView) parent.findViewById(R.id.seekBarTitle);
		txtTitle.setText(pref.getTitle());

		// Set the seekbar properties
		final SeekBar seekBar = (SeekBar) parent.findViewById(R.id.seekBar);
		seekBar.setMax(pref.getMaxValue());
		seekBar.setProgress(pref.getCurrentValue());
		// Set a change listener
		seekBar.setOnSeekBarChangeListener(new OnSeekBarChangeListener()
		{

			@Override
			public void onStopTrackingTouch(SeekBar seekBar)
			{
			}

			@Override
			public void onStartTrackingTouch(SeekBar seekBar)
			{
			}

			@Override
			public void onProgressChanged(SeekBar seekBar, int progress, boolean fromUser)
			{
				MyLog.d("GameMenuUiHelper.createSeekBarPreferenceFrame: onProgressChanged: " + progress);
				
				// Update the preference value
				pref.setCurrentValue(progress);
			}
		});

		// Music volume seekbar should be disabled if music mode is "none"
		if (pref.getKey().equals("pref_music_volume"))
		{
			sMusicVolSeekbar = seekBar;

			SharedPreferences prefs = PreferenceManager.getDefaultSharedPreferences(context);
			if (prefs.getString("pref_music_mode", "original").equals("none"))
			{
				seekBar.setEnabled(false);
			}
		}

		// Speech volume seekbar should be disabled if voice mode is "none"
		if (pref.getKey().equals("pref_speech_volume"))
		{
			sSpeechVolSeekbar = seekBar;

			SharedPreferences prefs = PreferenceManager.getDefaultSharedPreferences(context);
			if (prefs.getString("pref_voice_mode", "voice_subtitles").equals("subtitles"))
			{
				seekBar.setEnabled(false);
			}
		}

		// Subtitle speed seekbar should be disabled if voice mode "voice only"
		if (pref.getKey().equals("pref_subtitle_speed"))
		{
			sSubtitleSpeedSeekbar = parent.findViewById(R.id.seekBarFrame);

			SharedPreferences prefs = PreferenceManager.getDefaultSharedPreferences(context);
			if (!prefs.getString("pref_voice_mode", "voice_subtitles").equals("subtitles"))
			{
				sSubtitleSpeedSeekbar.setVisibility(View.INVISIBLE);
				sSubtitleSpeedSeekbar.setEnabled(false);
			}
		}
	}

	private static SeekBar sMusicVolSeekbar = null;
	private static SeekBar sSpeechVolSeekbar = null;
	private static View sSubtitleSpeedSeekbar = null;

}
